from python_sdk_remote.mini_logger import MiniLogger
from python_sdk_remote.utilities import our_get_env

from .Connector import get_connection

# TODO: We should move this code to python-sdk/infrastructure repo.
#  We should call our_python_init() which calls get_debug() as we might want to add things in the future
os_debug = our_get_env('DEBUG') or "False"
debug = os_debug.lower() == 'true' or os_debug == '1'

if debug:
    MiniLogger.info("Writer.py debug is on debug=" + str(debug))


class Writer:
    @staticmethod
    def add_message(message, log_level):  # TODO: not used. Can we / shall we have one add_message...()  method?
        if debug:
            MiniLogger.info("add_message" + message + ' ' + str(log_level))
        connection = None
        try:
            # creating connection
            connection = get_connection(schema_name="logger")
            cursor = connection.cursor()
            query = (f"INSERT INTO logger.logger_table (message, severity_id) "
                     f"VALUES ('{message}', {log_level})")
            cursor.execute(query)
        except Exception as e:
            MiniLogger.exception("Exception Writer.py Writer.add_message caught" + str(e))
        finally:
            if connection:
                connection.commit()

    # TODO We prefer to have one INSERT to the logger_table
    # INSERT to logger_table should be disabled by default and activated using combination of json and Environment variable enabling INSERTing to the logger_table
    # This function is called `if self.write_to_sql and self.debug_mode.is_logger_output(
    #                               self.component_id, LoggerOutputEnum.MySQLDatabase, message_severity.value)`
    @staticmethod
    def add_message_and_payload(message: str = None, **kwargs):
        connection = None
        try:
            connection = get_connection(schema_name="logger")
            params_to_insert = kwargs['object']
            cursor = connection.cursor()
            cursor.execute(
                f"INSERT INTO location.location_table (coordinate) "
                f"VALUES (POINT({params_to_insert.get('latitude') or 0},{params_to_insert.get('longitude') or 0}));")
            coordinate_id = cursor.lastrowid

            params_to_insert.pop('latitude', None)
            params_to_insert.pop('longitude', None)

            params_to_insert['location_id'] = coordinate_id
            listed_values = list(params_to_insert.values())
            joined_keys = ','.join(list(params_to_insert.keys()))
            if message is not None:
                listed_values.append(message)
                joined_keys += (',' if joined_keys else '') + 'message'

            placeholders = ','.join(['%s'] * len(listed_values))
            query = f"INSERT INTO logger.logger_table ({joined_keys}) VALUES ({placeholders})"
            cursor = connection.cursor()
            cursor.execute(query, listed_values)
        except Exception as e:
            MiniLogger.exception("Exception logger Writer.py add_message_and_payload " + str(e))
        finally:
            if connection:
                connection.commit()



from python_sdk_remote.mini_logger import MiniLogger

from .Connector import get_connection

cache = {}


class Component:
    @staticmethod
    def getDetailsByComponentId(component_id: int | str) -> tuple[str, str, str, str, str] | None:
        if component_id in cache:
            return cache[component_id]
        try:
            connection = get_connection(schema_name="component")
            cursor = connection.cursor()
            sql_query = ("SELECT name, component_type, component_category, testing_framework, api_type "
                         "FROM component.component_table WHERE component_id = %s")
            cursor.execute(sql_query, (component_id,))
            result = cursor.fetchone()
            cache[component_id] = result
            return result
        except Exception as e:
            MiniLogger.exception("getDetailsByComponentId Exception: " + str(e))



from python_sdk_remote.mini_logger import MiniLogger

from .Connector import get_connection
from .MessageSeverity import MessageSeverity
from .SendToLogzIo import SendTOLogzIo

COMPUTER_LANGUAGE = "Python"
COMPONENT_ID = 102
COMPONENT_NAME = 'Logger Python'
Logzio_handler = SendTOLogzIo()

cache = []


class Fields:
    @staticmethod
    def getFieldsSingelton():
        global cache
        if cache:
            return cache
        sql_query = f"DESCRIBE logger.logger_table"
        try:
            object1 = {
                'record': {'severity_id': MessageSeverity.Information.value,
                           'severity_name': MessageSeverity.Information.name, 'component_id': COMPONENT_ID,
                           'component_name': COMPONENT_NAME, 'computer_language': COMPUTER_LANGUAGE,
                           'message': "get_logger_table_fields activated"},
                'severity_id': MessageSeverity.Information.value,
                'component_id': COMPONENT_ID,
                'severity_name': MessageSeverity.Information.name,
                'component_name': COMPONENT_NAME,
                'COMPUTER_LANGUAGE': COMPUTER_LANGUAGE,
                'message': "get_logger_table_fields activated",
            }
            Logzio_handler.send_to_logzio(object1)
            con = get_connection(schema_name="logger")
            cursor = con.cursor()
            cursor.execute(sql_query)
            columns_info = cursor.fetchall()
            columns = [column[0] for column in columns_info]
            cache = columns
            return columns

        except Exception as e:
            MiniLogger.exception("logger-local-python-package LoggerService.py sql(self) Exception caught SQL=" +
                                 sql_query, e)
            return



from functools import wraps

from .Logger import Logger


def log_function_execution(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        logger_local = Logger.create_logger(**kwargs)
        object1 = {
            'args': str(args),
            'kwargs': str(kwargs),
        }
        logger_local.start(object=object1)
        result = func(*args, **kwargs)  # Execute the function
        object2 = {
            'result': result,
        }
        logger_local.end(object=object2)
        return result

    return wrapper
